@@ -1,5 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import hashlib
 from decimal import Decimal
 from collections import namedtuple, defaultdict
 from itertools import groupby
@@ -71,12 +72,9 @@ class Statement(Workflow, ModelSQL, ModelView):
     _number_depends = _depends + ['validation']
 
     name = fields.Char('Name', required=True)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, states=_states, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ],
-        depends=_depends)
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True,
+        states=_states, depends=_depends)
     journal = fields.Many2One('account.statement.journal', 'Journal',
         required=True, select=True,
         domain=[
@@ -423,7 +421,7 @@ class Statement(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def view_attributes(cls):
-        return [
+        return super().view_attributes() + [
             ('/tree', 'visual', If(Eval('state') == 'cancelled', 'muted', '')),
             ]
 
@@ -499,19 +497,34 @@ class Statement(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Line = pool.get('account.statement.line')
         Warning = pool.get('res.user.warning')
-
+        paid_cancelled_invoice_lines = []
         for statement in statements:
             getattr(statement, 'validate_%s' % statement.validation)()
+            paid_cancelled_invoice_lines.extend(l for l in statement.lines
+                if l.invoice and l.invoice.state in {'cancelled', 'paid'})
+
+        if paid_cancelled_invoice_lines:
+            warning_key = ('%s.statement_paid_cancelled_invoice_lines' %
+                    hashlib.md5(str(paid_cancelled_invoice_lines).encode(
+                        'utf-8')).hexdigest())
+            if Warning.check(warning_key):
+                raise StatementValidateWarning(warning_key,
+                    gettext('account_statement'
+                        '.msg_statement_invoice_paid_cancelled'))
+            Line.write(paid_cancelled_invoice_lines, {
+                    'invoice': None,
+                    })
 
         cls.create_move(statements)
 
         cls.write(statements, {
                 'state': 'validated',
                 })
-        common_lines = Line.search([
+        common_lines = [l for l in Line.search([
                 ('statement.state', '=', 'draft'),
-                ('invoice.state', '=', 'paid'),
+                ('invoice.state', 'in', ['posted', 'paid']),
                 ])
+            if l.invoice.reconciled]
         if common_lines:
             warning_key = '_'.join(str(l.id) for l in common_lines)
             if Warning.check(warning_key):
@@ -681,7 +694,11 @@ def origin_mixin(_states, _depends):
         currency_digits = fields.Function(fields.Integer(
                 "Currency Digits"), 'on_change_with_currency_digits')
         party = fields.Many2One(
-            'party.party', "Party", states=_states, depends=_depends)
+            'party.party', "Party", states=_states,
+            context={
+                'company': Eval('company', -1),
+                },
+            depends=_depends + ['company'])
         account = fields.Many2One(
             'account.account', "Account",
             domain=[
@@ -696,6 +713,11 @@ def origin_mixin(_states, _depends):
         description = fields.Char(
             "Description", states=_states, depends=_depends)
 
+        @classmethod
+        def __setup__(cls):
+            super().__setup__()
+            cls.__access__.add('statement')
+
         @classmethod
         def get_statement_states(cls):
             pool = Pool()
@@ -748,6 +770,7 @@ class Line(
         depends=['company'])
     invoice = fields.Many2One('account.invoice', 'Invoice',
         domain=[
+            ('company', '=', Eval('company', -1)),
             If(Bool(Eval('party')), [('party', '=', Eval('party'))], []),
             If(Bool(Eval('account')), [('account', '=', Eval('account'))], []),
             If(Eval('statement_state') == 'draft',
@@ -756,7 +779,7 @@ class Line(
             ],
         states=_states,
         context={'with_payment': False},
-        depends=['party', 'account'] + _depends)
+        depends=['company', 'party', 'account'] + _depends)
     origin = fields.Many2One('account.statement.origin', 'Origin',
         readonly=True,
         states={
@@ -977,6 +1000,7 @@ class Line(
             second_currency = None
 
         return MoveLine(
+            origin=self,
             description=self.description,
             debit=amount < zero and -amount or zero,
             credit=amount >= zero and amount or zero,
@@ -1012,6 +1036,7 @@ class LineGroup(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(LineGroup, cls).__setup__()
+        cls.__access__.add('statement')
         cls._order.insert(0, ('date', 'DESC'))
 
     @classmethod
